clear all; clc; close all;
warning off
%---------------------------------------------------------------------
%---------------------------------------------------------------------

% Demo for training the convolutional regressor for nuclei segmenation
% Input: 
%       All_Images:  grayscale images, for example, All_Images_10percent.mat
%       All_Masks:   binary masks for each grayscale image, for exmaple,
%                    All_Masks_10percent.mat
%       winsize:     the filter size, for example, winsize = 21
%       nbasis:      the number of filters to train, for example, nbasis =
%                    10
%       Maxite:      the maximum number of iterations (big loop) for running the entire
%                    training algorithm, for exmaple, Maxite = 50
%                    for exmaple, Maxite = 50
%       W_maxite:    the number of iterations for solving W 
%       W_SC_maxite: the number of iterations for solving sparse W
%       D_maxite:    the number of iterations for solving filter bank
%       attenuation: default 1 (no need to change)
%---------------------------------------------------------------------
%---------------------------------------------------------------------

%% Training
%------------------------------------------------------------------------
% Input 
load All_Images_10percent
load All_Masks_10percent
All_Images = All_Images(:,:,1:1);
All_Masks = All_Masks(:,:,1:1);

rr = randperm(size(All_Images,3));
winsize = 21;
nbasis = 10;
Maxite = 20;

W_maxite = 10;
W_SC_maxite = 10;
D_maxite = 10;
attenuation = 1;
%------------------------------------------------------------------------
% Give path for saving results at each iteration. In each iteration, we
% save linear combination coefficient vector W and filter bank D_old
Output_Folder = ['test'];
status = mkdir(Output_Folder);
if status
    disp('folder created')
else
    disp('error in creating folder')
end
addpath(Output_Folder)
%------------------------------------------------------------------------
%-------------------Preprocessing----------------------------------------
for ii = size(All_Images,3)
    All_Images(:,:,ii) = rescale(All_Images(:,:,ii),0,1); 
end

%------------------------------------------------------------------------
%--------------------Initialization--------------------------------------
D_0 = [];
tmp = randn(winsize*winsize,nbasis);
tmp = tmp * diag(1 ./ sqrt(sum(tmp.^2)));
for ii = 1:nbasis
    D_0 = cat(3,D_0,reshape(tmp(:,ii),winsize,winsize));    
end
d_old = D_0(:);
D_old = D_0;

%------------------------------------------------------------------------
%--------------------Convolutional Network Training Starts Here----------
W = zeros(nbasis,1);
segerror = zeros(Maxite,1);
residerr = zeros(Maxite,1);
All_Masks_Energy = sum(sum(sum(All_Masks(1+floor(winsize/2):end-floor(winsize/2),1+floor(winsize/2):end-floor(winsize/2),:).^2)));

for ite = 1:Maxite
%------------------------------------------------------------------------      
% Solve for W
%------------------------------------------------------------------------  
    addpath minFunc/
    woptions.Method = 'cg';%'lbfgs'; 
    woptions.maxIter = W_maxite;	  % Maximum number of iterations of L-BFGS to run 
    woptions.display = 'NONE';   

    Xconvolved_weighted_store = Compute_Convolution_Paral(All_Images,D_old,attenuation);
    funObj = @(ww)Obj_W_Attenuation_Group_faster_par(ww,Xconvolved_weighted_store,All_Masks,winsize);
    [W, f_val] = minFunc(funObj,W,woptions);
    
    addpath(genpath('L1General/'))
    lambdaVect = 0.0001 * ones(size(W));                        
    gOptions.maxIter = 200;
    gOptions.verbose = 0; % Set to 0 to turn off output
    goptions.corrections = 10; % Number of corrections to store for L-BFGS methods
    Xconvolved_weighted_store = Compute_Convolution_Paral(All_Images,D_old,attenuation);
    funObj = @(ww)Obj_W_Attenuation_Group_faster_par(ww,Xconvolved_weighted_store,All_Masks,winsize);
    W = myL1General2_PSSgb(funObj,W,lambdaVect,goptions,W_SC_maxite,1);
    [f_val] = Obj_W_Attenuation_Group_faster_par(W,Xconvolved_weighted_store,All_Masks,winsize);
    segerror(ite) = f_val;
    clear Xconvolved_weighted_store
%------------------------------------------------------------------------      
% Update Dictionary
%------------------------------------------------------------------------  
    addpath minFunc/
    doptions.Method = 'lbfgs'; 
    doptions.maxIter = D_maxite;	  % Maximum number of iterations of L-BFGS to run 
    doptions.display = 'NONE';   
    d_old = D_old(:);
    funObj2 = @(dd)Maskonly_Conv_Obj_Dictionary_Regressor_Group_Valid_Paral(All_Images,dd,All_Masks,W,attenuation);
    [d_old, f_val] = minFunc(funObj2,d_old,doptions);
    residerr(ite) = f_val;
    d_old = reshape(d_old,winsize^2,nbasis);
    d_old = d_old * diag(1 ./ sqrt(sum(d_old.^2)));
    d_old = d_old(:);

    D_old = reshape(d_old,winsize,winsize,nbasis);

    disp(['Iteration  ' num2str(ite) '   total segmentation error :  ' num2str(segerror(ite)/All_Masks_Energy) '   updated error ratio   :  ' num2str(residerr(ite)/All_Masks_Energy)])        
    save([Output_Folder '/Iteration_' num2str(ite) '_ratio_' num2str(residerr(ite)/All_Masks_Energy) '.mat'],'D_old','W');    
end

%% Visualize the learned filters
tmp = [];
for ii = 1:size(D_old,3)
    tt = flipud(fliplr(squeeze(D_old(:,:,ii))));
    tmp = [tmp tt(:)];
end
figure;
I = displayDictionaryElementsAsImage(tmp, floor(sqrt(nbasis)), floor(sqrt(nbasis)), winsize, winsize,0);
title('Updated dictionary atoms')
